## $Id: applicant.py 16481 2021-05-12 14:58:45Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

import grok
from zope.interface import implementedBy
from zope.component import getUtility
from waeup.kofa.interfaces import IExtFileStore
from waeup.kofa.applicants.applicant import ApplicantFactory
from waeup.kofa.utils.helpers import attrs_to_fields
from kofacustom.nigeria.applicants.applicant import NigeriaApplicant
from waeup.uniben.applicants.interfaces import(
    ICustomApplicant, ICustomUGApplicantEdit, ICustomPGApplicantEdit,
    IPUTMEApplicantEdit, ITranscriptApplicant)

class CustomApplicant(NigeriaApplicant):

    grok.implements(ICustomApplicant, ICustomUGApplicantEdit,
        ICustomPGApplicantEdit, IPUTMEApplicantEdit, ITranscriptApplicant)
    grok.provides(ICustomApplicant)

    applicant_graduated_mapping = [
        ('firstname', 'firstname'),
        ('middlename', 'middlename'),
        ('lastname', 'lastname'),
        ('sex', 'sex'),
        ('date_of_birth', 'date_of_birth'),
        ('email', 'email'),
        ('phone', 'phone'),
        ('matric_number', 'matric_number'),
        ]

    def admchecking_fee_paid(self):
        # We don't charge if admission checking fee is not set.
        session = str(self.__parent__.year)
        try:
            session_config = grok.getSite()['configuration'][session]
        except KeyError:
            return True
        if session_config.admchecking_fee == 0:
            return True
        for key in self.keys():
            ticket = self[key]
            if ticket.p_state == 'paid' and \
                ticket.p_category == 'admission_checking':
                return True
        return False

    def _copyPassportImage(self, student):
        """Copy any passport image over to student location. 
        """
        file_store = getUtility(IExtFileStore)
        appl_file = file_store.getFileByContext(self)
        if appl_file is None:
            return
        fname = 'passport.jpg'
        # In Uniben we save JAMB pictures as passport2.jpg.
        if getattr(self.__parent__, 'prefix', None) in ('ase', 'pude'):
            fname = 'passport2.jpg'
        stud_file_id = IFileStoreNameChooser(student).chooseName(
            attr=fname)
        file_store.createFile(stud_file_id, appl_file)
        return

# Set all attributes of CustomApplicant required in ICustomApplicant as field
# properties. Doing this, we do not have to set initial attributes
# ourselves and as a bonus we get free validation when an attribute is
# set.
CustomApplicant = attrs_to_fields(CustomApplicant)

class CustomApplicantFactory(ApplicantFactory):
    """A factory for customized applicants.
    """

    def __call__(self, *args, **kw):
        return CustomApplicant()

    def getInterfaces(self):
        return implementedBy(CustomApplicant)
